@@ -8,6 +8,7 @@ import inspect
 import logging
 import mimetypes
 from functools import partial, update_wrapper
+from collections import Iterable
 from urllib import parse
 import http.client
 from http.cookies import SimpleCookie
@@ -54,6 +55,10 @@ class App:
 
         self.provide('config', load_config)
 
+        @self.provide('json_encoder')
+        def json_encoder():
+            return JSONEncoder
+
         def route(method, pattern, handler=None):
             if handler is None:
                 return partial(self.provider.router.append, method, pattern)
@@ -129,7 +134,7 @@ class App:
         if body is None:
             body = ''
         if type(body) not in [str, bytes]:
-            body = json.dumps(body)
+            body = json.dumps(body, cls=self.provider.json_encoder)
             headers = {'Content-Type': 'application/json; charset=utf-8'}
         _headers.update(headers)
         if code == 200 and is_fresh(self.provider.environ, _headers):
@@ -248,6 +253,22 @@ class App:
         self.provider.router.append('GET', re.compile(
             "^" + url_root + "(?P<url>.+)$"), static_handler(fs_root))
 
+    def json_encode(self, t, encoder=None):
+        """specify custom json encode method
+
+        Example:
+
+            @json_enocode(ObjectId)
+            def encode_objectid(obj):
+                return str(obj)
+        """
+        if encoder is None:
+            return partial(self.json_encode, t)
+        self.provider.json_encoder.add_encoder(t, encoder)
+
+    def __repr__(self):
+        return repr(self.provider.router)
+
 
 class Provider:
 
@@ -514,8 +535,6 @@ class RestfulRouter:
         ('GET',    '%(path)s/<%(id)s>/edit', 'edit'),
     ]
 
-    restful_methods = [method for _, _, method in restful_routes]
-
     def __init__(self, router):
         self.router = router
 
@@ -565,10 +584,9 @@ class RestfulRouter:
         else:
             fns = get_methods(module)
 
-        custom_rules = [(getattr(fn, '__httpmethod__', 'GET'),
-                        '%s/<%s>/%s' % (url_path, url_id, fn.__name__),
-                        fn) for fn in fns
-                        if fn.__name__ not in self.restful_methods]
+        custom_rules = [(fn.__httpmethod__,
+                         '%s/<%s>/%s' % (url_path, url_id, fn.__name__),
+                        fn) for fn in fns]
         self.router.extend(rules)
         self.router.extend(custom_rules)
         return module
@@ -581,6 +599,23 @@ class Response:
         self.headers = headers
 
 
+class JSONEncoder(json.JSONEncoder):
+
+    __custom_encoders__ = {}
+
+    @classmethod
+    def add_encoder(self, t, encode):
+        self.__custom_encoders__[t] = encode
+
+    def default(self, obj):
+        for t in self.__custom_encoders__:
+            if isinstance(obj, t):
+                return self.__custom_encoders__[t](obj)
+        if isinstance(obj, Iterable):
+            return list(obj)
+        return super().default(self, obj)
+
+
 def invoke(fn, *param_dicts):
     "call a function with a list of dicts providing params"
     prepared_params = {}
@@ -693,16 +728,17 @@ def is_fresh(request_headers, response_headers):
 
 
 def get_module_fns(module):
-    "get defined functions of a module"
+    "get request handlers of a module"
     attrs = [getattr(module, a) for a in dir(module) if not a.startswith('_')]
     return [attr for attr in attrs if isinstance(attr, types.FunctionType)
-            and attr.__module__ == module.__name__]
+            and hasattr(attr, '__httpmethod__')]
 
 
 def get_methods(cls):
-    "get public methods of a class"
+    "get request handlers of a class"
     attrs = [getattr(cls, a) for a in dir(cls) if not a.startswith('_')]
-    return [attr for attr in attrs if isinstance(attr, types.FunctionType)]
+    return [attr for attr in attrs if isinstance(attr, types.FunctionType)
+            and hasattr(attr, '__httpmethod__')]
 
 
 def method(httpmethod):
@@ -713,3 +749,16 @@ def method(httpmethod):
         fn.__httpmethod__ = httpmethod.upper()
         return fn
     return add_method
+
+
+def cache_control(*args, **kwargs):
+    """decorator to specify Cache-Control header
+
+    Example:
+
+        @cache_control('public', max-age=60)
+        @get('/')
+        def home():
+            pass
+    """
+    pass
